@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # coding: utf-8
 
-# In[4]:
+# In[2]:
 
 
 try:
@@ -12,10 +12,10 @@ except:
         pass
 
 
-# In[5]:
+# In[1]:
 
 
-version = '0.78'
+version = '0.81'
 
 import re
 import tempfile
@@ -698,7 +698,9 @@ def imwrite(path, img):
     os.makedirs(os.path.dirname(path), exist_ok=True)
     cv2.imwrite(path, img[:, :, [2, 1, 0]])
     
-def imjoin(imgs, axis=1):
+def imjoin(imgs, axis=1, pad=0):
+    if pad > 0:
+        imgs = [impad(img, right=pad) for img in imgs]
     return np.concatenate(imgs, axis=axis)
 
 def impad(img, top=0, bottom=0, left=0, right=0, color=255):
@@ -723,21 +725,21 @@ def imshow(array, scale=1):
 # In[67]:
 
 
-if is_testing():
-    img = imread('http://via.placeholder.com/70.png')
-    imshow(img)
-
-
-# In[ ]:
-
-
-if is_testing():
-    imshow('http://via.placeholder.com/70.png')
+#if is_testing():
+#    img = imread('http://via.placeholder.com/70.png')
+#    imshow(img)
 
 
 # In[68]:
 
 
+#if is_testing():
+#    imshow('http://via.placeholder.com/70.png')
+
+
+# In[69]:
+
+
 def imscaleNN(img, s):
     return cv2.resize(img, None, fx=s, fy=s, interpolation=cv2.INTER_NEAREST)
 
@@ -745,7 +747,7 @@ def imscaleBic(img, s):
     return cv2.resize(img, None, fx=s, fy=s, interpolation=cv2.INTER_CUBIC)
 
 
-# In[69]:
+# In[70]:
 
 
 if is_testing():
@@ -764,7 +766,7 @@ if is_testing():
         imshow(out_large_interbic)
 
 
-# In[70]:
+# In[71]:
 
 
 def imRepeat(img, height, width):
@@ -778,7 +780,7 @@ def imRepeat(img, height, width):
     return img
 
 
-# In[71]:
+# In[72]:
 
 
 if is_testing():
@@ -786,33 +788,33 @@ if is_testing():
     imshow(imRepeat(img_random_color_3x5, 7, 11), scale=10)
 
 
-# In[72]:
+# In[73]:
 
 
 def imNewWhite(height, width):
     return np.ones((height, width, 3)).astype(np.uint8) * 255
 
 
-# In[73]:
+# In[74]:
 
 
 if is_testing(): imshow(imNewWhite(10, 500) - 30)
 
 
-# In[74]:
+# In[75]:
 
 
 def imNewBlack(height, width):
     return np.zeros((height, width, 3)).astype(np.uint8)
 
 
-# In[75]:
+# In[76]:
 
 
 if is_testing(): imshow(imNewBlack(10, 500))
 
 
-# In[76]:
+# In[77]:
 
 
 def imAddNoiseGauss(img, std):
@@ -821,10 +823,10 @@ def imAddNoiseGauss(img, std):
     noisy = (np.clip(img.astype(np.float) + noise.astype(np.float), 0, 255)).astype(np.uint8)
     return noisy
 
-if is_testing(): imshow(imAddNoiseGauss(imread('http://via.placeholder.com/150.png'), 10))
+if is_testing(): imshow(imAddNoiseGauss(imNewWhite(150, 150), 30))
 
 
-# In[ ]:
+# In[78]:
 
 
 def imJpgDegradation(img, quality):
@@ -834,10 +836,10 @@ def imJpgDegradation(img, quality):
     decimg = cv2.imdecode(encimg, 1)
     return decimg
 
-if is_testing(): imshow(imJpgDegradation(imread('http://via.placeholder.com/150.png'), 15))
+#if is_testing(): imshow(imJpgDegradation(imread('http://via.placeholder.com/150.png'), 15))
 
 
-# In[ ]:
+# In[79]:
 
 
 def imCropCenter(img, size):
@@ -852,35 +854,35 @@ def imCropCenter(img, size):
     return img[h_start:h_end, w_start:w_end]
 
 
-# In[ ]:
+# In[80]:
 
 
-if is_testing(): 
-    imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 10))
+#if is_testing(): 
+#    imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 10))
 
 
-# In[ ]:
+# In[81]:
 
 
-if is_testing(): 
-    imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 49))
+#if is_testing(): 
+#    imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 49))
 
 
-# In[ ]:
+# In[82]:
 
 
-if is_testing(): 
-    imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 50))
+#if is_testing(): 
+#    imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 50))
 
 
-# In[ ]:
+# In[83]:
 
 
-if is_testing(): 
-    imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 100))
+#if is_testing(): 
+#    imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 100))
 
 
-# In[77]:
+# In[84]:
 
 
 def imCropRandom(img, size, auto_size=True):
@@ -903,17 +905,17 @@ def imCropRandom(img, size, auto_size=True):
     return img[h_start:h_end, w_start:w_end]
 
 
-# In[ ]:
+# In[85]:
 
 
-if is_testing(): 
-    imshow(imCropRandom(imread('http://via.placeholder.com/40.png'), 10))
-    imshow(imCropRandom(imread('http://via.placeholder.com/40.png'), 39))
-    imshow(imCropRandom(imread('http://via.placeholder.com/40.png'), 40))
-    imshow(imCropRandom(imread('http://via.placeholder.com/40.png'), 41))
+#if is_testing(): 
+#    imshow(imCropRandom(imread('http://via.placeholder.com/40.png'), 10))
+#    imshow(imCropRandom(imread('http://via.placeholder.com/40.png'), 39))
+#    imshow(imCropRandom(imread('http://via.placeholder.com/40.png'), 40))
+#    imshow(imCropRandom(imread('http://via.placeholder.com/40.png'), 41))
 
 
-# In[78]:
+# In[86]:
 
 
 def imGallery(imgs, pad=0):
@@ -944,7 +946,7 @@ def imGallery(imgs, pad=0):
     return out
 
 
-# In[79]:
+# In[87]:
 
 
 if is_testing():
@@ -952,57 +954,57 @@ if is_testing():
     listOfDummyImagesBlackToWhite = [imNewBlack(10, 10) + 25 * i for i in range(10)]
 
 
-# In[80]:
+# In[88]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:1]))
 
 
-# In[81]:
+# In[89]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:2]))
 
 
-# In[82]:
+# In[90]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:2], pad=1))
 
 
-# In[83]:
+# In[91]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:5], pad=1))
 
 
-# In[84]:
+# In[92]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:9], pad=1))
 
 
-# In[85]:
+# In[93]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:10], pad=1))
 
 
-# In[86]:
+# In[95]:
 
 
-if is_testing():
-    img40x20 = imread('http://via.placeholder.com/40x20.png')
-    imshow(imGallery([img40x20, img40x20, img40x20], pad=1))
+#if is_testing():
+#    img40x20 = imread("http://via.placeholder.com/40x20.png")
+#    imshow(imGallery([img40x20, img40x20, img40x20], pad=1))
 
 
-# In[87]:
+# In[96]:
 
 
 def imAddTextHeader(img, text, h=None, size=None):
@@ -1039,7 +1041,7 @@ if is_testing():
 
 # # pytorch
 
-# In[88]:
+# In[97]:
 
 
 def ptT(img_np_uint8):
@@ -1056,7 +1058,7 @@ def ptUint8(img_tensor):
     
 
 
-# In[89]:
+# In[98]:
 
 
 if is_testing():
@@ -1068,14 +1070,14 @@ if is_testing():
 
 # # Files
 
-# In[90]:
+# In[99]:
 
 
 def fiFindByWildcard(wildcard):
     return natsort.natsorted(glob.glob(wildcard, recursive=True))
 
 
-# In[91]:
+# In[100]:
 
 
 if is_testing():
@@ -1104,7 +1106,7 @@ if is_testing():
         imshow(imGallery(imgs))
 
 
-# In[92]:
+# In[101]:
 
 
 def fiRemove(path):
@@ -1132,7 +1134,7 @@ def fiAgeHours(path):
     return (time.time() - os.stat(path)[stat.ST_MTIME]) / 3600
 
 
-# In[93]:
+# In[102]:
 
 
 if is_testing():
@@ -1164,7 +1166,7 @@ if is_testing():
         print("files: " + "\n".join(fiFindByWildcard(os.path.join(d, "**", "*"))))
 
 
-# In[94]:
+# In[103]:
 
 
 def fiUnzip(zip_path, out_dir=None, remove=False):
@@ -1179,7 +1181,7 @@ def fiUnzip(zip_path, out_dir=None, remove=False):
         zip.extractall(out_dir)
 
 
-# In[95]:
+# In[104]:
 
 
 def fiJoin(*args):
@@ -1191,7 +1193,7 @@ if is_testing():
 
 # # Jupyter
 
-# In[96]:
+# In[105]:
 
 
 def jnSettings():
@@ -1202,7 +1204,7 @@ if is_testing():
     jnSettings()
 
 
-# In[97]:
+# In[106]:
 
 
 def jnNbFullWidth():
@@ -1216,7 +1218,7 @@ def jnPandasSettings():
     pd.set_option('display.max_colwidth', 100000)
 
 
-# In[98]:
+# In[107]:
 
 
 def jnImageRenderingPixelated():
@@ -1228,7 +1230,7 @@ def jnImageRenderingAuto():
     display(HTML('<style type = text/css> img { image-rendering: auto; } </style>'))
 
 
-# In[99]:
+# In[108]:
 
 
 def jnDefault():
@@ -1246,7 +1248,7 @@ if is_testing():
 
 
 
-# In[100]:
+# In[109]:
 
 
 def jnHtmlRow(strList, width=100):
@@ -1259,7 +1261,7 @@ if is_testing():
     
 
 
-# In[101]:
+# In[110]:
 
 
 def jnImgToHtmlB64(img):
@@ -1294,7 +1296,7 @@ def jnImageRow(imgs, names=None, width=100, external=False, title_img=None, titl
     display(HTML(html))
 
 
-# In[ ]:
+# In[111]:
 
 
 if is_testing():
@@ -1302,7 +1304,7 @@ if is_testing():
    
 
 
-# In[ ]:
+# In[112]:
 
 
 if is_testing():
@@ -1312,7 +1314,7 @@ if is_testing():
     jnImageRow(imgs, external=True, title='test')
 
 
-# In[ ]:
+# In[113]:
 
 
 
@@ -1322,7 +1324,7 @@ if is_testing():
 
 # # Shell
 
-# In[ ]:
+# In[114]:
 
 
 def shell(cmd):
@@ -1337,7 +1339,7 @@ def shell(cmd):
 
 # # GIT
 
-# In[ ]:
+# In[115]:
 
 
 def gitGetHash():
@@ -1347,7 +1349,7 @@ if is_testing():
     print(gitGetHash())
 
 
-# In[ ]:
+# In[116]:
 
 
 def gitGetBranch():
@@ -1357,7 +1359,7 @@ if is_testing():
     print(gitGetBranch())
 
 
-# In[ ]:
+# In[117]:
 
 
 def gitGetMessage():
@@ -1367,7 +1369,7 @@ if is_testing():
     print(gitGetMessage())
 
 
-# In[ ]:
+# In[118]:
 
 
 def gitGetAllFiles():
@@ -1377,7 +1379,7 @@ if is_testing():
     print(gitGetAllFiles())
 
 
-# In[ ]:
+# In[119]:
 
 
 def gitListByTime(path=None):
@@ -1393,7 +1395,7 @@ if is_testing():
     print(gitListByTime()[-3:])
 
 
-# In[ ]:
+# In[120]:
 
 
 def gitTime(path):
@@ -1404,18 +1406,18 @@ if is_testing():
     print(gitTime("__init__.py"))
 
 
-# In[ ]:
+# In[121]:
 
 
 def gitTimeRaw(path):
-    _time = lambda x: shell('git log -1 --format="%ad" --date=raw -- {filename}'.format(filename=x)).replace("\n", "")
+    _time = lambda x: shell('cd $(dirname {filename}) && git log -1 --format="%ad" --date=raw -- {filename}'.format(filename=x)).replace("\n", "")
     return _time(path)
     
 if is_testing():
     print(gitTimeRaw("__init__.py"))
 
 
-# In[ ]:
+# In[122]:
 
 
 def shellSortDirsBySize():
@@ -1427,14 +1429,14 @@ if is_testing():
 
 # # Package Files
 
-# In[ ]:
+# In[123]:
 
 
 if is_testing():
     package_files = {}
 
 
-# In[ ]:
+# In[124]:
 
 
 def shell(cmd):
@@ -1442,7 +1444,7 @@ def shell(cmd):
     return result.stdout.decode()
 
 
-# In[ ]:
+# In[125]:
 
 
 if is_testing():
@@ -1474,7 +1476,7 @@ setup(
 )""".format(requirements=", ".join(["'" + r + "'" for r in requirements]), version=version)
 
 
-# In[ ]:
+# In[126]:
 
 
 if is_testing():
@@ -1484,7 +1486,7 @@ description-file = README.md
 """
 
 
-# In[ ]:
+# In[127]:
 
 
 if is_testing():
@@ -1507,14 +1509,14 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE."""
 
 
-# In[ ]:
+# In[128]:
 
 
 if is_testing():
     package_files['README.md'] = ""
 
 
-# In[ ]:
+# In[129]:
 
 
 if is_testing():
@@ -1523,7 +1525,7 @@ if is_testing():
     
 
 
-# In[ ]:
+# In[130]:
 
 
 if is_testing():
@@ -1534,7 +1536,7 @@ lx/__init__.py
 """
 
 
-# In[ ]:
+# In[131]:
 
 
 if is_testing():
@@ -1545,7 +1547,7 @@ if is_testing():
         print("\n".join(fiFindByWildcard(os.path.join(d, '**/*'))))
         
         output = shell("""python3 -c 'import importlib; lx = importlib.import_module(name=".", package="lx"); print(lx.version)'""")
-        print(output.strip(), 'Version correct: ', output.strip() == version)
+        print(output.strip(), 'Version correct: ', output.strip() == version, "(", output.strip(), version, ")")
         
         if output.strip() == version:
             #print(shell("cd {d} && git status && git config user.name 'lx' && git config user.email 'lx' && git commit -a -m 'Add' && git log && git push && python setup.py sdist && twine upload dist/* --verbose".format(d=d)))
@@ -1575,3 +1577,9 @@ if is_testing():
 
 
 
+
+# In[ ]:
+
+
+
+
